#!/bin/python3

"""
USAGE:
"""
def usage():
	print("python3 feedback_to_js.py <FeedbackDatei>")
	print("python3 feedback_to_js.py --help")
	print("")
	print("Das Ergebnis ist JavascriptCode, der in der WebConsole des Browsers eingetragen werden muss (Anleitung am Ende der Datei bzw. nach erfolgreicher Konvertierung.")
	print("Feedbackdatei sollte im JSON Format sein. In Etwa:")
	print("[")
	print("  {")
	print('    "Typ": "1",')
	print('    "Pos": "3",')
	print('    "Seite": "25"')
	print('    "Text": "Feedbackbox 1"')
	print('   },')
	print("  {")
	print('    "Typ": "0",')
	print('    "Pos": "2",')
	print('    "Seite": "111"')
	print('    "Text": "Feedbackbox 1"')
	print('    "Text": "Feedbackbox 2"')
	print('   }')
	print(']')

import json
import sys


error = 0
def fail(m):
	print("// Error:", m, file=sys.stderr)
	global error
	error += 1

js_code = []

def get_art(t):
	if t in ["Sprache", "0"]:
		return 0
	elif t in ["Layout", "1"]:
		return 1
	elif t in ["Inhalt", "2"]:
		return 2
	elif t in ["Berechnung", "Referenz", "3"]:
		return 3
	elif t in ["Sonstiges", "4"]:
		return 4
	else:
		return -1

def get_pos(p):
	if p in ["einspalting", "0"]:
		return 0
	elif p in ["links", "1"]:
		return 1
	elif p in ["rechts", "2"]:
		return 2
	elif p in ["beide", "3"]:
		return 3
	else:
		return -1

def handle_fb(fb, idx):
	global js_code
	if not isinstance(fb, dict):
		fail(f"Parserfehler: Eintrag {idx+1} ist kein Objekt")
		return
	for p in ["Seite", "Pos", "Typ", "Text"]:
		if not p in fb:
			fail(f"Eintrag {p} erforderlich für Feedback {idx+1}")
			return
	errPos = f"Eintrag {idx+1} (S. {fb['Seite']})"
	failed = False

	# generate js
	js_tmp = []
	if len(js_code) != 0: # the first is already present
		js_tmp += ["gf_addPart();"] # new feedback
	# Fehlerart
	art = get_art(fb["Typ"])
	if art == -1:
		fail(f"Typ {fb['Typ']} unbekannt ({errPos}). Benutze Sprache, Layout, Inhalt, Berechnung oder Sonstiges")
		failed = True
	js_tmp += [f"document.getElementsByName('type')[{idx}].selectedIndex = {art};"] # works because indices are 0..
	js_tmp += [f"gf_changeType('{art}', {idx+1});"]
	# Seite
	try:
		seite = int(fb["Seite"])
	except:
		fail(f"Seite ist keine Zahl für Feedback {errPos}")
		failed = True
	seite = fb["Seite"]
	js_tmp += [f"document.getElementsByName('page')[{idx}].value = \"{seite}\";"]
	# Position
	pos = get_pos(fb["Pos"])
	if pos == -1:
		fail(f"Position von {errPos} unbekannt. Benutze einspalting, links, rechts oder beide")
		failed = True
	js_tmp += [f"document.getElementsByName('column')[{idx}].selectedIndex = {pos};"]
	# Text
	text1 = fb["Text"].strip().replace("\n", "\\n").replace("\"", "\\\"")
	if text1 == "":
		fail(f"Kein Text für {errPos} gegeben.")
		failed = True
	js_tmp += [f"document.getElementsByName('text1')[{idx}].value = \"{text1}\";"]
	if art in [0, 3]:
		if not "Text2" in fb or fb["Text2"].strip() == "":
			fail(f"Typ von {errPos} benötigt zusätzlich einen zweiten Text \"Text2\"")
			failed = True
		else:
			text2 = fb["Text2"].strip().replace("\n", "\\n").replace("\"", "\\\"")
			# need complex DOM moving, because number of text2 elements is unknown
			js_tmp += [f"document.getElementsByName('text1')[{idx}].parentNode.nextSibling.children[1].value = \"{text2}\";"]
	if failed:
		return
	# all ok, add to global
	js_code += js_tmp

def handle_json(j):
	for i in range(len(j)):
		handle_fb(j[i], i)

def main():
	if len(sys.argv) != 2 or sys.argv[1] == "--help":
		usage()
		return
	with open(sys.argv[1]) as inf:
		j = json.load(inf)
	handle_json(j)
	print("\n".join(js_code))
	print("// " + "-"*15)
	print("// Feedback eintragen:")
	print("// >>> Mach die Seite zum Eintragen von Feedback auf (die Seite muss komplett unverändert sein, also am besten neu laden")
	print("// >>> Öffne die Console des Browsers (F12 drücken, dann in dem aufgehenden Bereich den Reiter 'Console' wählen. Das Fenster sollte weiß sein bis auf ein '>>'. Dahinter kann Text eingegeben werden)")
	print("// >>> Kopiere den Text von oben dorthin und drücke ENTER")
	print("// >>> Kontrolliere, ob alles gut aussieht")
	print("// >>> Feedback abschicken")
	if error > 0:
		print("// ==========")
		print(f"// Aber {error} Fehler sind aufgetreten. Die entsprechenden Einträge werden fehlen")

if __name__ == "__main__":
	main()
